
#include "actionseditor.h"

#include <qlayout.h>
#include <qobjectlist.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qtable.h>
#include <qsettings.h>

#include "images.h"

#define COL_ICON 0
#define COL_NAME 1
#define COL_DESC 2
#define COL_SHORTCUT 3


ActionsEditor::ActionsEditor(QWidget * parent, const char * name, bool modal, WFlags f)
	: QDialog(parent, name, modal, f)
{
    actionsTable = new QTable(0, COL_SHORTCUT + 1, this);

    actionsTable->horizontalHeader()->setLabel(COL_ICON, ""); //tr("Icon"));
    actionsTable->horizontalHeader()->setLabel(COL_NAME, tr("Name"));
    actionsTable->horizontalHeader()->setLabel(COL_DESC, tr("Description"));
    actionsTable->horizontalHeader()->setLabel(COL_SHORTCUT, tr("Shortcut"));

	actionsTable->setColumnReadOnly(COL_ICON, true);
	actionsTable->setColumnReadOnly(COL_NAME, true);
	actionsTable->setColumnReadOnly(COL_DESC, true);

	actionsTable->setColumnStretchable(COL_ICON, false);
	actionsTable->setColumnStretchable(COL_NAME, true);
	actionsTable->setColumnStretchable(COL_DESC, true);

    actionsTable->verticalHeader()->hide();
    actionsTable->setLeftMargin(0);
    actionsTable->setColumnReadOnly(0, true);

    QPushButton *okButton = new QPushButton(tr("&OK"), this);
    QPushButton *cancelButton = new QPushButton(tr("&Cancel"), this);

    connect(actionsTable, SIGNAL(currentChanged(int, int)),
            this, SLOT(recordAction(int, int)));
    connect(actionsTable, SIGNAL(valueChanged(int, int)),
            this, SLOT(validateAction(int, int)));
    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

    QHBoxLayout *buttonLayout = new QHBoxLayout;
    buttonLayout->setSpacing(8);
    buttonLayout->addStretch(1);
    buttonLayout->addWidget(okButton);
    buttonLayout->addWidget(cancelButton);

    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->setMargin(8);
    mainLayout->setSpacing(8);
    mainLayout->addWidget(actionsTable);
    mainLayout->addLayout(buttonLayout);

    setCaption(tr("Edit Actions"));
}

ActionsEditor::~ActionsEditor() {
}

void ActionsEditor::addActions(QWidget *widget) {
	QACTION *action;

#if QT_VERSION < 0x040000
	QObjectList *actions = widget->queryList("QAction");
	action = static_cast<QACTION*>(actions->first());

    while (action) {
        actionsList.append(action);
		action = static_cast<QAction*>(actions->next());
    }
#else
	QList<Q3Action *> actions = widget->findChildren<Q3Action *>();
	for (int n=0; n < actions.count(); n++) {
		action = static_cast<QACTION*> (actions[n]);
        actionsList.append(action);
    }
#endif

	updateView();
}

void ActionsEditor::updateView() {
	actionsTable->setNumRows( actionsList.count() );

    QACTION *action;
	for (int n=0; n < actionsList.count(); n++) {
		action = static_cast<QACTION*> (actionsList[n]);
		QPixmap p = action->iconSet().pixmap();
		if (!p.isNull()) p = Images::resize(&p, 16);
		actionsTable->setPixmap(n, COL_ICON, p );
		actionsTable->setText(n, COL_NAME, action->name());
        actionsTable->setText(n, COL_DESC, action->text());
        actionsTable->setText(n, COL_SHORTCUT, QString(action->accel()));
	}
	actionsTable->adjustColumn( COL_ICON );
	actionsTable->adjustColumn( COL_NAME );
	actionsTable->adjustColumn( COL_DESC );
}


void ActionsEditor::applyChanges() {
	for (int row = 0; row < (int)actionsList.size(); ++row) {
		QACTION *action = actionsList[row];
		action->setAccel(QKeySequence(actionsTable->text(row, COL_SHORTCUT)));
	}
}

void ActionsEditor::accept() {
	applyChanges();
    QDialog::accept();
}

void ActionsEditor::recordAction(int row, int column)
{
	if (column == COL_SHORTCUT) {
	    oldAccelText = actionsTable->item(row, column)->text();
	}
}

void ActionsEditor::validateAction(int row, int column)
{
	if (column == COL_SHORTCUT) {
	    QTableItem *item = actionsTable->item(row, column);
	    QString accelText = QString(QKeySequence(item->text()));

	    if (accelText.isEmpty() && !item->text().isEmpty())
	        item->setText(oldAccelText);
	    else
	        item->setText(accelText);
	}
}


void ActionsEditor::saveToConfig(QObject *o, QSettings *set) {
	qDebug("ActionsEditor::saveToConfig");

	set->beginGroup("actions");

	QACTION *action;
#if QT_VERSION < 0x040000
	QObjectList *actions = o->queryList("QAction");
	action = static_cast<QACTION*>(actions->first());

	while (action) {
		QString accelText = QString(action->accel());
		set->writeEntry(action->name(), accelText);
		action = static_cast<QAction*>(actions->next());
	}
#else
	QList<Q3Action *> actions = o->findChildren<Q3Action *>();
	for (int n=0; n < actions.count(); n++) {
		action = static_cast<QACTION*> (actions[n]);
		QString accelText = QString(action->accel());
		set->writeEntry(action->name(), accelText);
    }
#endif

	set->endGroup();
}


void ActionsEditor::loadFromConfig(QObject *o, QSettings *set) {
	qDebug("ActionsEditor::loadFromConfig");

	set->beginGroup("actions");

	QACTION *action;
#if QT_VERSION < 0x040000
	QObjectList *actions = o->queryList("QAction");
	action = static_cast<QACTION*>(actions->first());

	while (action) {
		QString accelText = set->readEntry(action->name());
		action->setAccel(QKeySequence(accelText));
		action = static_cast<QAction*>(actions->next());
	}
#else
	QList<Q3Action *> actions = o->findChildren<Q3Action *>();
	for (int n=0; n < actions.count(); n++) {
		action = static_cast<QACTION*> (actions[n]);
		QString accelText = set->readEntry(action->name());
		action->setAccel(QKeySequence(accelText));
    }
#endif

	set->endGroup();
}

/*
void ActionsEditor::saveToConfig(QSettings * set) {
	set->beginGroup("actions");

	QACTION *action;
	for (int n=0; n < actionsList.count(); n++) {
		action = static_cast<QACTION*> (actionsList[n]);
		QString accelText = QString(action->accel());
		set->writeEntry(action->name(), accelText);
	}

	set->endGroup();
}

void ActionsEditor::loadFromConfig(QSettings * set) {
	set->beginGroup("actions");

	QACTION *action;
	for (int n=0; n < actionsList.count(); n++) {
		action = static_cast<QACTION*> (actionsList[n]);
		QString accelText = set->readEntry(action->name());
		action->setAccel(QKeySequence(accelText));
	}

	set->endGroup();
}
*/
